// ============================================================================================
// Datei:      main.cpp
// Desc:       main file
// Coder:      Kongo
// ============================================================================================


// ============================================================================================
// Libs
// ============================================================================================

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <tchar.h>
#include <mmsystem.h>


#include "zed4d.h"
#include "zhelp.h"
#include "terrain.h"
#include "main.h"

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Variablen
// ============================================================================================

HWND           g_hMainWnd;
HINSTANCE      g_hMainInstance;
BOOL           g_bWindowActive = false;
BOOL           g_bGamedown = false;
BOOL           g_bGameGo = false;

zD3DManager    Manager;
zD3DCamera     Cam;
zMouse         Mouse;
zKeyboard      Keyboard;
zD3DFont       Font;
zTimer         Timer;

zTerrain       Terrain;

// --------------------------------------------------------------------------------------------


// ============================================================================================
// Funktionen
// ============================================================================================

bool Game_Init(VOID *Data)
{
   if (!Manager.Init(g_hMainWnd,g_hMainInstance,WIDTH,HEIGHT,BPP,ZPP,SPP,false,false,NULL)) {
      OutputDebugString("Manager.Init failed\n");
      return false;
   }

   // Projektionsmatrix: View Frustum 45 Grad
   D3DXMATRIX matrix;
   D3DXMatrixPerspectiveFovLH(&matrix, 0.8f, 4.0f / 3.0f, 1.0f, 100.0f);
   pDev->SetTransform(D3DTS_PROJECTION, &matrix);

   Mouse.Init(g_hMainInstance,g_hMainWnd);
   Keyboard.Init(g_hMainInstance,g_hMainWnd);
   Mouse.Activate();
   Keyboard.Activate();

   Font.Init("Arial",12);
   Font.Restore();

   Cam.Activate(0.0f,10.0f,0.0f);
   Cam.vDir.x = 10.0f;
   Cam.vDir.y = 10.0f;
   Cam.vDir.z =  7.0f;

   _setrs(D3DRS_CULLMODE,D3DCULL_NONE);



   Terrain.Create("heightmap.raw");



   g_bGameGo = true;
   return true;
}

// --------------------------------------------------------------------------------------------

bool Game_Main(VOID *Data)
{
   static DWORD oldtime;
   DWORD time;
   int x,y,z;
   static float forward;

   time = timeGetTime();
   time = time - oldtime;
   oldtime = timeGetTime();
   Timer.Frame();

   Mouse.Read();
   Keyboard.Read();
  
   Mouse.GetPosChange(x,y,z);
   Cam.Rotate(0.0005f*y*time,0.0005f*x*time,0.0005f*z*time);

   if (Keyboard.IsDown(DIK_W))
      forward += time*0.0005f;
   if (Keyboard.IsDown(DIK_S))
      forward -= time*0.0005f;
   forward = _max(forward,0.0f);
   forward = _min(forward,0.7f);

   Cam.Translate(0.0f,0.0f,forward);

   Cam.Update();

   _clear(D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,D3DCOLOR_XRGB(0,0,0),1.0f,0);






   TCHAR strFPS[15] = _T("FPS: ");
   TCHAR strstr[30] = _T("");
   TCHAR cpos[60] = _T("Position: x = ");
   _tcscat(cpos, _gcvt(Cam.vPos.x,4,strstr));
   _tcscat(cpos, _T(", y = "));
   _tcscat(cpos, _gcvt(Cam.vPos.y,4,strstr));
   _tcscat(cpos, _T(", z = "));
   _tcscat(cpos, _gcvt(Cam.vPos.z,4,strstr));
   _tcscat(strFPS, _gcvt(Timer.GetAverageFramesPerSec(),4,strstr));
   _beginscene();
   Terrain.Draw(Cam.vPos);
   Font.DrawText(10,10,0xffffffff,cpos);
   Font.DrawText(10,24,0xffffffff,strFPS);
   Font.DrawText(10,38,0xffffffff,"Keys:  f/g: Shade;  q/e: Fill;  b/n/m: Cull\0");
   _endscene();

   _present();

   return true;
}


// --------------------------------------------------------------------------------------------

bool Game_Shutdown(VOID *Data)
{



   Terrain.Destroy();



   Font.Shutdown();
   Keyboard.Shutdown();
   Mouse.Shutdown();
   Manager.Shutdown();

   g_bGameGo = false;
   return true;
}

// --------------------------------------------------------------------------------------------

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg) {
      case WM_ACTIVATEAPP:
         g_bWindowActive = (bool) wParam;
        return 0;
      case WM_DESTROY:
         g_bGamedown = true;
         PostQuitMessage(0);
        return 0;
      case WM_KEYDOWN:
         if (wParam == 'F')
            _setrs(D3DRS_SHADEMODE,D3DSHADE_FLAT);
         if (wParam == 'G')
            _setrs(D3DRS_SHADEMODE,D3DSHADE_GOURAUD);
         if (wParam == 'Q')
            _setrs(D3DRS_FILLMODE,D3DFILL_SOLID);
         if (wParam == 'E')
            _setrs(D3DRS_FILLMODE,D3DFILL_WIREFRAME);
         if (wParam == 'B')
            _setrs(D3DRS_CULLMODE,D3DCULL_CW);
         if (wParam == 'N')
            _setrs(D3DRS_CULLMODE,D3DCULL_NONE);
         if (wParam == 'M')
            _setrs(D3DRS_CULLMODE,D3DCULL_CCW);

         if (wParam == VK_ESCAPE) {
            PostMessage(hWnd,WM_CLOSE,0,0);
            return 0;
         }
        break;
   }
   return DefWindowProc(hWnd,msg,wParam,lParam);
}

// --------------------------------------------------------------------------------------------

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   WNDCLASSEX     winclass;
   MSG            msg;
   HWND           hWnd;

   winclass.cbSize            = sizeof(WNDCLASSEX);
   winclass.style             = CS_HREDRAW | CS_VREDRAW;
   winclass.lpfnWndProc       = WinProc;
   winclass.cbClsExtra        = 0;
   winclass.cbWndExtra        = 0;
   winclass.hInstance         = hInstance;
   winclass.hIcon             = LoadIcon(NULL, IDI_APPLICATION);
   winclass.hCursor           = LoadCursor(NULL, IDC_ARROW); 
   winclass.hbrBackground     = (HBRUSH)GetStockObject(BLACK_BRUSH);
   winclass.lpszMenuName      = NULL;
   winclass.lpszClassName     = WINDOW_CLASS_NAME;
   winclass.hIconSm           = LoadIcon(NULL, IDI_APPLICATION);

   if (!RegisterClassEx(&winclass))
      return -1;

   if (!(hWnd = CreateWindowEx(  WS_EX_TOPMOST,
                                 WINDOW_CLASS_NAME,
                                 WINDOW_TITLE,
                                 WS_POPUP | WS_VISIBLE,
                                 0,0,
                                 GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN),
                                 NULL,
                                 NULL,
                                 hInstance,
                                 NULL)))
      return -2;

   g_hMainWnd = hWnd;
   g_hMainInstance = hInstance;

   UpdateWindow(hWnd);

   if (!Game_Init(NULL))
      return -3;

   while (!g_bGamedown) {
      if (PeekMessage(&msg,hWnd,0,0,PM_REMOVE)) {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      }

      if (g_bWindowActive && g_bGameGo) {
         if (!Game_Main(NULL)) {
            g_bGameGo = false;
            PostMessage(hWnd,WM_CLOSE,0,0);
         }
      }
   }

   Game_Shutdown(NULL);
   return msg.wParam;
}
